// constest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#pragma comment (lib, "..\\_lib\\boxdoio.lib")
#pragma comment (lib, "..\\_lib\\bxdclbr.lib")
#pragma comment (lib, "..\\..\\common\\lib\\unilptx.lib")

#include "..\..\common\include\devtypes.h"
#include "..\..\common\include\unilptx.h"
#include "..\_include\doiointf.h"
#include "..\_include\bxdclbr.h"


const int nVersionsCount = 4;
LPCTSTR PLDResNames[nVersionsCount] = {"121", "122","122","122"};

/* Voltage Range (8 div, 1-2-5 stepping)
idx		screen		range
 0		 2 mV/div	16 mV
 1		 5 mV/div	40 mV
 2		10 mV/div	80 mV
 ...
 9		 2  V/div	16 V
10		 5  V/div	40 V
*/
#define nVoltageRangeIdx	8

/* sample rate (1-2-5 stepping)
idx		screen		frequency	sample
   RIS MODE
-6		 10 ns/div	 10 GHz		100 ps
-5		 20 ns/div	  5 GHz		200 ps
...
-1		500 ns/div	200 MHz		  5 ns
   REAL MODE
 0		  1 us/div	100 MHz		 10 ns
 1		  2 us/div	 50 MHz		 20 ns
...
21		 10  s/div	 10  Hz		100 ms
*/
#define nFrequencyIdx		0

#define fVoltageOffset		0.0

int main(int argc, char* argv[])
{
	printf("\n=========================================================");
	printf("\nUNIPRO Digital Oscilloscope sample (C)AURIS, 2004");
	printf("\n=========================================================\n\n");
	
	// Finding device
	printf("Trying to find and lock device...\t\t");
	CDeviceID* pID=UNILPTDrv::LockDevice(dtOscilloscope);
	if (pID) {
		ULONG nVer = pID->m_nDeviceVersion;
		ULONG nSer = nVer & 0xFFFF;
		nVer = (nVer>>24)*100 + pID->m_DeviceType * 10 + ((nVer>>16) & 0xFF);
		printf("Board ID: B%d.%d\n", nVer, nSer);
	}
	else {
		printf("failed\n");
		return -1;
	}

	// Initializing device calibration data
	printf("Initializing device calibration data...\t\t");
	CBoxDOCalibrationData cd;
	if (InitBoxDOCalibrationData(cd, TRUE))
		printf("done\n");
	else {
		printf("failed\n");
		return -2;
	}

	// Creating device control data
	COscilloscopeIOControlInfo oscinfo;	
	memset(&oscinfo, 0, sizeof(oscinfo));
	
	try
	{
		// Device configuration
		printf("Configuring device PLD...\t\t\t");
		int nPLDResIndex = ((pID->m_nDeviceVersion >> 16) & 0xff) - 1;
		BOOL bPrgOK = (nPLDResIndex >= 0 || nPLDResIndex < nVersionsCount);
		if (bPrgOK)
			if (!BoxDOIOControl(*pID, IOCTL_PLDPROGRAMMING, (void*)PLDResNames[nPLDResIndex]))
				throw "PLD configuration failed";
		printf("done\n");

		// reading device calibration data
		printf("Reading device calibration data...\t\t");
		int nSC = SerializeBoxDOCalibrationData(*pID, cd, FALSE);
		if (!nSC) printf("done\n");
		else printf("failed\n");
		if (nSC & 1)
			throw "error reading internal calibrator data";
		if (nSC & 2){
			printf("Gain calibration data corrupted. Performing calibration.\n");
			if (!CalibrateGain(*pID, cd, -1, -1)) throw "Gain calibration failed";}
		if (nSC & 4){
			printf("Offset calibration data corrupted. Performing calibration.\n");
			if (!CalibrateZeroLevel(*pID, cd, -1, -1)) throw "Offset calibration failed";}
		if (nSC & 8){
			printf("Trigger calibration data corrupted. Performing calibration.\n");
			if (!CalibrateTriggerLevel(*pID, cd, -1, -1)) throw "Trigger calibration failed";}
		if (nSC & 16){
			printf("RIS calibration data corrupted. Performing calibration.\n");
			if (!CalibrateRIS(*pID, cd)) throw "RIS calibration failed";}

		// initializing device control data
		printf("Initializing device control data...\t\t");
		InitOscInfo(oscinfo, cd, TRUE, nVoltageRangeIdx, nFrequencyIdx);
		printf("done\n");

		// initializing channels' parameters
		printf("Initializing channels' parameters...\t\t");
		for (int i=0; i<BoxDOChannelsCount; i++){
			CChannelIOControlInfo& ch = oscinfo.m_Channels[i];
			ch.m_ChannelState = csEnabled; // channel enabled
			ch.m_nVolts = nVoltageRangeIdx; // gain
			ch.m_nZeroLevel = ZeroLevelToCode(fVoltageOffset, i, ch.m_nVolts, cd, NULL); // offset
			ch.m_VerticalCoupling = vcDC; // DC coupling
		}
		// turning off calibration mode
		oscinfo.m_bAdjustment = FALSE;
		oscinfo.m_bAdjustmentNotReadMemory = FALSE;
		oscinfo.m_bAdjustmentCalibratorMeander = FALSE;
		oscinfo.m_CalibratorType = ctOff;// calibrator is off
		// 
		oscinfo.m_nNumberChannels = 2; // 2 channels
		oscinfo.m_nFrequency = nFrequencyIdx;
		oscinfo.m_bInterleaving = (nFrequencyIdx < 0);
		// data size
		oscinfo.m_nSegmentSizeCode = 10; // 2^10 - 2 = 1022 bytes of valid data;
		// trigger settings
		oscinfo.m_bTriggerNormal = FALSE; // mode = auto, so the next are don't care
		oscinfo.m_nPrePostTrigger = 0;
		oscinfo.m_TriggerSource = tsExternal;
		oscinfo.m_TriggerCoupling = tcDC;
		oscinfo.m_TriggerType = ttSimple;
		oscinfo.m_bTriggerSlopeNegative = FALSE;
		oscinfo.m_nTriggerLevel = TriggerLevelToCode(0, 0, oscinfo.m_TriggerSource, oscinfo.m_TriggerCoupling, 0, cd, NULL);
		// advanced (smart) trigger setting
		oscinfo.m_bTimeTriggerGreat = FALSE; 
		oscinfo.m_fTimeTriggerLength = 40e-9;
		oscinfo.m_fTimeTriggerFactor = 20e-9;
		oscinfo.m_TimeTriggerMode = ttmPositiveSlopePeriod;
		// TV trigger settings
		oscinfo.m_TVTriggerMode = tvmField1;
		oscinfo.m_nTVTriggerString = 1;
		oscinfo.m_bTVTriggerNTSC = FALSE;
		printf("done\n");

		// applying device settings
		printf("Applying device settings...\t\t\t");
		if(!BoxDOIOControl(*pID, IOCTL_RESET, &oscinfo)) 
			throw "cannot apply device settings (IOCTL_RESET failed)";
		printf("done\n");

		// starting acquisition process
		printf("Starting acquisition process...\t\t\t");
		if(!BoxDOIOControl(*pID, IOCTL_RUN, &oscinfo)) // Start
			throw "cannot start aquisition (IOCTL_RUN failed)";
		printf("done\n");
		
		// waiting for "ready" flag
		printf("Waiting for acquisition is complete...\t\t");
		oscinfo.m_bReady = FALSE;
		do
		{
			if(!BoxDOIOControl(*pID, IOCTL_READY, &oscinfo))
				throw "cannot get device status (IOCTL_READY failed)";
			::Sleep(10); // sleep to free CPU for other applications
		}
		while (!oscinfo.m_bReady);
		printf("done\n");

		// Reading the channels' data from device to oscinfo.m_Channels[i].m_pData[]		
		printf("Reading the channels' data...\t\t\t");
		if(!BoxDOIOControl(*pID, IOCTL_READ, &oscinfo))
			throw "Read failed";
		printf("done\n");

		// calculating valid samples count (last two samples may be invalid)
		int nValidSamplesCount = ((1 << oscinfo.m_nSegmentSizeCode) - 2); 

		//////////////////////////////////////////////////////////////////////////
		// any data processing here

		// converting codes to voltage values and calculating average voltage
		for (int nChanIdx=0; nChanIdx<2; nChanIdx++)
		{
			CChannelIOControlInfo& ch = oscinfo.m_Channels[nChanIdx];
			int nZero = BoxDOADCCodeMax / 2;
			double fCoef = 8/*div*/ * BoxDOVoltsValue(ch.m_nVolts)/*V/div*/ / (BoxDOADCCodeMax+1);
			ch.m_fTrueZeroLevel = CodeToZeroLevel(ch.m_nZeroLevel,nChanIdx,ch.m_nVolts,cd);
			double* pVoltsCh0 = new double[nValidSamplesCount];
			double fSum = 0;
			for (int nSampleIdx=0; nSampleIdx<nValidSamplesCount; nSampleIdx++){
				pVoltsCh0[nSampleIdx] = (nZero - ch.m_pData[nSampleIdx]) * fCoef - ch.m_fTrueZeroLevel;
				fSum += pVoltsCh0[nSampleIdx];
			}
			delete [] pVoltsCh0;
			fSum /= nValidSamplesCount;
			printf("Average voltage on signal input %d is\t\t %+5g\n", nChanIdx, fSum);
		}

		// Do not call IOCTL_RESET when repeat aquisition with the same parameters. Save your time.
		// Correct cycle is IOCTL_RUN + IOCTL_READY + IOCTL_READ.

		printf("Complete.\n");

	}

	catch (LPCTSTR lpszErr)
	{		
		printf("\nError: %s\n", lpszErr);
	}

	// unlocking device
	if (pID)
	{
		UNILPTDrv::UnLockDevice(*pID);
		pID = NULL;
	}

	// device control info deinitialization (dealocating memory etc.)
	ClearOscInfo(oscinfo);

	printf("Press 'Enter' to exit...");
	getchar();
	return 0;		
}
